package cz.uhk.wallboard.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.servlet.ModelAndView;

import cz.uhk.wallboard.model.TypZpravy;
import cz.uhk.wallboard.model.Zprava;
import cz.uhk.wallboard.security.Operation;
import cz.uhk.wallboard.security.OperationManager;
import cz.uhk.wallboard.security.UlozZpravuOperation;
import cz.uhk.wallboard.services.ZpravaService;

@Controller
@RequestMapping("/zprava")
public class ZpravaController {
	private OperationManager operationManager;

	// @Autowired
	// HttpServletRequest request;

	private ZpravaService zpravaService;

	@Autowired
	public ZpravaController(OperationManager operationManager, ZpravaService zpravaService) {
		// TODO idealne by se mela kontrolovat nenullovost (vsech zavislosti)
		// i kdyz v nasem pripade Spring null nevyda a selže dřív
		this.operationManager = operationManager;
		this.zpravaService = zpravaService;
	}

	@RequestMapping(value = { "/", "" }, method = RequestMethod.GET)
	public ModelAndView vypisViditelneZpravy() throws Exception {
		ModelAndView mav = new ModelAndView("seznamZprav");
		List<Zprava> viditelneZpravy = zpravaService.getViditelneZpravy();
		mav.addObject("zpravy", viditelneZpravy);
		return mav;
	}

	@RequestMapping(value = { "{id:\\d+}", }, method = RequestMethod.GET)
	public ModelAndView detailZpravy(@PathVariable Integer id) throws Exception {

		operationManager.requestOperation("/zprava/{id:\\d+},GET");
		ModelAndView mav = new ModelAndView("detailZpravy");
		mav.addObject("zprava", new Zprava(1, "detail zpravy " + id, "obshah zpravy " + id, TypZpravy.LEKTORSKA));
		return mav;
	}

	@RequestMapping(value = { "{typZpravy}/{nazev}" }, method = RequestMethod.GET)
	public ModelAndView novaZprava(@PathVariable String nazev, @PathVariable String typZpravy) throws Exception {
		ModelAndView mav = new ModelAndView("novaZprava");
		Zprava novaZprava = new Zprava(null, nazev, "", TypZpravy.valueOf(typZpravy.toUpperCase()));
		mav.addObject("zprava", novaZprava);
		return mav;
	}

	@RequestMapping(value = { "{typZpravy}/{nazev}" }, method = RequestMethod.POST)
	public ModelAndView novaZprava(@PathVariable String nazev, String obsah, @PathVariable String typZpravy)
			throws Exception {
		Operation operace = operationManager
				.getOperation(String.format("/zprava/%s/%s,%s", typZpravy, nazev, RequestMethod.POST));
		Zprava novaZprava = new Zprava(null, nazev, obsah, TypZpravy.valueOf(typZpravy.toUpperCase()));
		((UlozZpravuOperation) operace).setUkladanaZprava(novaZprava);
		operationManager.requestOperation(operace);

		return new ModelAndView("redirect:/");
	}
}
