package cz.uhk.wallboard.dal.impl;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import cz.uhk.wallboard.dal.ZpravaDao;
import cz.uhk.wallboard.model.TypZpravy;
import cz.uhk.wallboard.model.Zprava;

@Repository
public class ZpravaDaoImpl implements ZpravaDao {

	private static final String selectPodleId = "SELECT * FROM zprava WHERE id = ?";
	private static final String insertZprava = "INSERT INTO ZPRAVA (ID, AUTOR, OBSAH, NAZEV, TYP, KDY_VYTVORENO) VALUES (zprava_id_seq.nextval, ?, ? ,? ,?, ?)";
	private static final String selectNejnovejsiZpravaPodleNazvu = "select * from Zprava where id = (select max(id) from zprava where nazev = ?)";
	private static final String selectNejnovejsiZpravy = "select * from Zprava where id in (select max(id) from Zprava group by nazev, typ)";
	private DataSource dataSource;

	@Autowired
	public ZpravaDaoImpl(DataSource dataSource) {
		this.dataSource = dataSource;
	}

	@Override
	public Zprava zpravaPodleId(int id) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public List<Zprava> getNejnovejsiZpravy() {
		List<Zprava> vysledek = new ArrayList<>();
		try (Connection connection = dataSource.getConnection()) {
			PreparedStatement preparedStatement = connection.prepareStatement(selectNejnovejsiZpravy);
			ResultSet resultSet = preparedStatement.executeQuery();
			while (resultSet.next()) {
				vysledek.add(mapRow(resultSet));
			}
		} catch (SQLException e) {
			throw new RuntimeException(e);
		}
		return vysledek;
	}

	@Override
	public Zprava getNejnovejsiZpravaPodleNazvu(String nazev) {
		Zprava vysledek = null;
		try (Connection connection = dataSource.getConnection()) {
			PreparedStatement preparedStatement = connection.prepareStatement(selectNejnovejsiZpravaPodleNazvu);
			preparedStatement.setString(1, nazev);
			ResultSet resultSet = preparedStatement.executeQuery();
			while (resultSet.next()) {
				vysledek = mapRow(resultSet);
			}
		} catch (SQLException e) {
			throw new RuntimeException(e);
		}
		return vysledek;
	}

	private Zprava mapRow(ResultSet resultSet) throws SQLException {
		Zprava vysledek;
		String nazev = resultSet.getString("nazev");
		String obsah = resultSet.getString("obsah");
		int id = resultSet.getInt("id");
		String typ = resultSet.getString("typ");
		TypZpravy typZpravy = TypZpravy.valueOf(typ);
		vysledek = new Zprava(id, nazev, obsah, typZpravy);
		return vysledek;
	}

	@Override
	public int insert(Zprava vkladana, String usernameAutora, Date casVzniku) {
		try (Connection connection = dataSource.getConnection()) {
			PreparedStatement preparedStatement = connection.prepareStatement(insertZprava);
			preparedStatement.setString(1, usernameAutora);
			preparedStatement.setString(2, vkladana.getObsah());
			preparedStatement.setString(3, vkladana.getNazev());
			preparedStatement.setString(4, vkladana.getTypZpravy().name());
			preparedStatement.setDate(5, new java.sql.Date(casVzniku.getTime()));
			preparedStatement.executeUpdate();
		} catch (SQLException e) {
			throw new RuntimeException(e);
		}
		return 0; // TODO zjistit id noveho radku (pry to nejak jde) nebo nevracet nic, nepotrebuju to, zatim
	}

}
