package cz.uhk.wallboard.model;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

/**
 * 
 * Statistika, kterou počítá aplikace z jiných statistik (tedy nepřijde z venku, protože jí stat server nesbírá)
 * 
 * Ekvivalence nezávisí na zdrojových statistikách
 */
public class OdvozenaStatistika extends Statistika {
	private List<? extends Statistika> zdrojoveStatistiky;

	public OdvozenaStatistika(String nazev, double hodnota,
			Collection<? extends Statistika> zdrojoveStatistiky) {
		super(nazev, hodnota);
		List<Statistika> defenzivniKopie = new ArrayList<Statistika>();
		for (Statistika statistika : zdrojoveStatistiky) {
			defenzivniKopie.add(statistika);			
		}
		this.zdrojoveStatistiky = Collections.unmodifiableList(defenzivniKopie);
	}

	@Override
	public String toString() {
		return "OdvozenaStatistika [zdrojoveStatistiky=" + getZdrojoveStatistiky() + ", getHodnota()=" + getHodnota()
				+ ", getNazev()=" + getNazev() + ", getObsah()=" + getObsah() + "]";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = super.hashCode();
		result = prime * result + ((getZdrojoveStatistiky() == null) ? 0 : getZdrojoveStatistiky().hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (!super.equals(obj))
			return false;
		if (getClass() != obj.getClass())
			return false;
		OdvozenaStatistika other = (OdvozenaStatistika) obj;
		if (getZdrojoveStatistiky() == null) {
			if (other.getZdrojoveStatistiky() != null)
				return false;
		} else if (!getZdrojoveStatistiky().equals(other.getZdrojoveStatistiky()))
			return false;
		return true;
	}
	
	/**
	 * 
	 * @return neměnitelnou kopii statistik, ze kterých byla tato vypočítána.
	 */
	public List<? extends Statistika> getZdrojoveStatistiky() {
		return zdrojoveStatistiky;
	}	
}
