package cz.uhk.wallboard.security;

import java.io.IOException;
import java.util.List;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;

import org.springframework.security.core.context.SecurityContextHolder;

import waffle.spring.NegotiateSecurityFilter;

/**
 * Do SecurityContext kontextu, vloží Authentication s daným uživatelem a danými rolemi.
 */
public class DummyNegotiateSecurityFilter extends NegotiateSecurityFilter {

	private String principalName;
	private List<String> roles;
	private final boolean developmentMode;

	public DummyNegotiateSecurityFilter() {
		this.developmentMode = "true".equalsIgnoreCase(System.getProperty("developmentMode", "false"));
	}

	public void setPrincipalName(String principalName) {
		this.principalName = principalName;
	}

	public void setRoles(List<String> roles) {
		this.roles = roles;
	}

	@Override
	public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain)
			throws IOException, ServletException {
		if (developmentMode) {

			RunAsAuthentication authentication = new RunAsAuthentication(principalName,
					roles.toArray(new String[roles.size()]));

			// nastaveni prihlaseni uzivatele do SpringSecurity
			SecurityContextHolder.getContext().setAuthentication(authentication);

			// pokracovani ve vykonavani filtru, jinak by doslo k zastaveni
			chain.doFilter(request, response);
		} else {
			throw new IllegalStateException(
					"Při filtrování požadavku byl použit " + DummyNegotiateSecurityFilter.class.getClass().getName()
							+ " ale není nastavena volba JVM developmentMode na true.");
		}
	}

}
