package cz.uhk.wallboard.security;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.stereotype.Component;

import cz.uhk.wallboard.model.exception.NeznamaOperaceException;

@Component
public class OperationManager {

	private Map<String, Operation> zabezpeceneOperace;

	@Autowired
	public OperationManager(Collection<? extends Operation> zabezpeceneOperace) {
		this.zabezpeceneOperace = new HashMap<String, Operation>();
		for (Operation operation : zabezpeceneOperace) {
			this.zabezpeceneOperace.put(operation.getName(), operation);
		}
	}

	public void requestOperation(Operation pozadovanaOperace) {
		if (pozadovanaOperace.isAuthorized()) {
			pozadovanaOperace.perform();
		} else {
			throw new AccessDeniedException(String.format("K této operaci nemáte oprávnění."));
		}
	}

	/**
	 * Metoda pro provedení operací, kterým není třeba nejdřív nastavit nějaká data.
	 * 
	 * @param nazevPozadovaneOperace
	 */
	public void requestOperation(String nazevPozadovaneOperace) {
		Operation operation = getOperation(nazevPozadovaneOperace);
		requestOperation(operation);
	}

	public Operation getOperation(String nazevOperace) {
		Operation operace = zabezpeceneOperace.get(nazevOperace);
		if (operace == null) {
			throw new NeznamaOperaceException(
					String.format("OperationManager %s nezná operaci %s", this, nazevOperace));
		}
		return operace.zkopirujSe();
	}

	@Override
	public String toString() {
		return "OperationManager [zabezpeceneOperace=" + zabezpeceneOperace + "]";
	}

}
