package cz.uhk.wallboard.security;

import java.util.ArrayList;
import java.util.Collection;

import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;

/**
 * Základní pomocná implementace Authentication, která má nastaveno pouze jméno (PrincipalName) a základní práva.
 * 
 * Použití: Instance se používá v jUnit testech, pokud se vyžaduje podvržení přihlašovacích práv pro daný test, nebo v Dummy SpringSecurity filtru po simulování různých práv pro
 * konkrétního uživatele.
 */
public class RunAsAuthentication implements Authentication {

	private static final long serialVersionUID = 5913888175085071134L;
	private boolean authenticated;
	private String name;
	private Collection<GrantedAuthority> grantedAuthorities;

	public RunAsAuthentication(String principalName, String[] roles) {
		this.name = principalName;
		grantedAuthorities = new ArrayList<GrantedAuthority>();

		if (roles != null) {
			for (String role : roles) {
				grantedAuthorities.add(new SimpleGrantedAuthority(role));
			}
		}

		this.authenticated = true;
	}

	@Override
	public String getName() {
		return name;
	}

	@Override
	public Collection<? extends GrantedAuthority> getAuthorities() {
		return grantedAuthorities;
	}

	@Override
	public Object getCredentials() {
		return null;
	}

	@Override
	public Object getDetails() {
		return null;
	}

	@Override
	public Object getPrincipal() {
		return name;
	}

	@Override
	public boolean isAuthenticated() {
		return authenticated;
	}

	@Override
	public void setAuthenticated(boolean isAuthenticated) throws IllegalArgumentException {
		this.authenticated = isAuthenticated;
	}

}
