package cz.uhk.wallboard.security;

import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.test.context.TestContext;
import org.springframework.test.context.support.AbstractTestExecutionListener;

/**
 * Listener pro jUnit testy, ktery podle anotace RunAsUser provede fiktivni prihlaseni (vyplni role) pro daneho uzivatele nad danou metodou.
 */
public class RunAsSupportTestExecutionListener extends AbstractTestExecutionListener {

	@Override
	public void beforeTestMethod(TestContext testContext) throws Exception {
		super.beforeTestMethod(testContext);
		RunAsUser runAsUser = testContext.getTestMethod().getAnnotation(RunAsUser.class);
		if (runAsUser != null) {
			// login
			SecurityContextHolder.getContext()
					.setAuthentication(new RunAsAuthentication(runAsUser.principalName(), runAsUser.roles()));
		}
	}

	@Override
	public void afterTestMethod(TestContext testContext) throws Exception {
		super.afterTestMethod(testContext);
		RunAsUser runAsUser = testContext.getTestMethod().getAnnotation(RunAsUser.class);
		if (runAsUser != null) {
			// logout
			SecurityContextHolder.getContext().setAuthentication(null);
		}
	}

}
