package cz.uhk.wallboard.services;

import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TimerTask;

import javax.servlet.ServletContext;

import org.codehaus.jackson.map.ObjectMapper;
import org.codehaus.jackson.type.TypeReference;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.context.WebApplicationContext;
import org.springframework.web.context.support.WebApplicationContextUtils;

import cz.uhk.wallboard.dal.StatistikaDto;
import cz.uhk.wallboard.dal.ZpravaDao;
import cz.uhk.wallboard.model.Prepocitavac;
import cz.uhk.wallboard.model.Statistika;
import cz.uhk.wallboard.model.WallboardovaPolozka;
import cz.uhk.wallboard.model.Zprava;

//TODO tohle se pry taky da springem tak, ze se udela nejaky scheduler,
//rekne se jak casto se ma volat a parametry jako url pak muze vlozit Spring
//viz. Casova osa
public class Task extends TimerTask {
	private static final Logger logger = LoggerFactory.getLogger(Task.class);

	ServletContext context;

	public Task(ServletContext context) {
		this.context = context;
		context.setAttribute("statistika", Collections.emptyMap());
	}

	// metoda kontaktuje aplikaci na statistiky a ziska jako odpoved Stringy s nazvem a hodnotou, ze kterych vyrobi statistiku
	@Override
	public void run() {
		try {
			ObjectMapper mapper = new ObjectMapper();

			// TODO komunikace s externim systemem - tohle by melo byt v DAO
			// nejdriv musim udelat spousteni tasku pres Spring
			String urlStatSenderu = context.getInitParameter("StatSenderUrl");
			URL url = new URL(urlStatSenderu);
			List<StatistikaDto> statistikyDto = mapper.readValue(url, new TypeReference<List<StatistikaDto>>() {
			});

			List<Statistika> statistiky = new ArrayList<Statistika>(statistikyDto.size());
			for (StatistikaDto statistikaDto : statistikyDto) {
				Statistika statistika = new Statistika(statistikaDto.nazev, statistikaDto.hodnota);
				statistiky.add(statistika);
			}

			Prepocitavac prepocitavac = new Prepocitavac(statistiky);
			Map<String, Statistika> prepocitanaStatistika = prepocitavac.vytvorOdvozeneStatistiky();
			// zarad do vyslednych statistik vsechny prepocitane
			Map<String, WallboardovaPolozka> vysledek = new HashMap<>(prepocitanaStatistika);

			// zarad do vyslednych statistik vsechny zdrojove
			for (Statistika statistika : statistiky) {
				vysledek.put(statistika.getNazev(), statistika);
			}

			// zarad do vyslednych statistik zpravy
			WebApplicationContext ctx = WebApplicationContextUtils.getWebApplicationContext(context);
			ZpravaDao zpravaDao = ctx.getBean(ZpravaDao.class);
			List<Zprava> zpravy = zpravaDao.getNejnovejsiZpravy();
			for (Zprava zprava : zpravy) {
				vysledek.put(zprava.getTypZpravy().getNazevBezDiakritiky() + "_" + zprava.getNazev(), zprava);
			}

			context.setAttribute("statistika", vysledek);

		} catch (Throwable e) {
			logger.error("Při načítání aktuálních statistik došlo k chybě.", e);
			// nechceme zobrazovat zastarala data, radsi nebudeme zobrazovat nic
			context.setAttribute("statistika", new HashMap<>());
		}

	}

}
